% "brelan" simule un joueur tentant d'avoir un brelan ; et répond true s'il
% a réussi, false sinon.
function reponse = brelan
% On va simuler les 8 premières cartes en haut du sabot, stockées dans
% "haut_sabot". On commence par allouer la mémoire à cet effet.
haut_sabot = zeros (1, 8);
% Dans un premier temps, les cartes sont codées par un chiffre entre 1 et
% 52. On crée le vecteur "restante" des cartes restant dans le sabot,
% "restante(i)" valant true tant que la carte numéro i n'a pas encore été
% vue en haut du paquet.
restante = true * ones (1, 52);
% On va tirer une par une les 8 cartes du haut du sabot.
for i = 1:8
    % Noter la syntaxe « while true » lorsque la condition de sortie de la
    % boucle n'est pas située à son point d'entrée, avec « break » ensuite
    % pour sortir de la boucle.
    while true
        % Pour trouver la prochaine carte, il faut faire un tirage uniforme
        % parmi les cartes n'ayant pas encore été tirées. La technique
        % adoptée ici consiste à employer la méthode de rejet: on fait un
        % tirage "carte_candidate" uniforme parmi {1, ..., 52}, qu'on
        % recommence éventuellement jusqu'à obtenir une valeur n'ayant pas
        % encore été tirée.
        carte_candidate = ceil (52 * rand ());
        % On teste si la carte candidate a déjà été tirée, et on procède en
        % conséquence.
        if restante(carte_candidate)
            restante(carte_candidate) = false;
            break
        end
    end
    % On stocke en mémoire l'identité de la carte qu'on vient de tirer.
    haut_sabot(i) = carte_candidate;
end
% En fait, à partir de maintenant les cartes ne m'intéressent qu'au travers
% de leurs hauteurs. La transformation que j'applique ici transforme 1, 2,
% 3 ou 4 en 1 ; 5, 6, 7 ou 8 en 2, etc.
haut_sabot = ceil (haut_sabot / 4);
% La main initiale du joueur est constituée par les 5 premières cartes du
% sabot.
main = haut_sabot(1:5);
% Le joueur va choisir quelles cartes il rejette parmi les 5 premières
% tirées. Pour cela, il doit commencer par compter le nombre de cartes de
% chaque hauteur: "nombre(h)" sera ainsi le nombre de cartes de hauteur h.
nombre = zeros (1, 13);
% On parcourtla liste des cartes en main (les 5 premières du sabot), et on
% met en jour "nombre" en conséquence.
for i = 1:5
    nombre(main(i)) = nombre(main(i)) + 1;
end
% "valeur_retenue" est la hauteur la plus souvent observée (tranchée
% arbitrairement en cas d'ex-æquo). On rejettera toutes les cartes n'ayant
% pas cette hauteur (dans la limite des 3 cartes autorisées). Noter qu'on
% utilise la syntaxe à deux arguments de la fonction "max", le second
% argument indiquant l'indice pour lequel le maximum est atteint ; comme on
% n'utilise pas la première valeur retournée par la fonction "max", on
% utilise la syntaxe « ~ » pour jeter cette valeur à la poubelle.
[~, valeur_retenue] = max (nombre);
% "position_sabot" indique la position de la prochaine carte à tirer dans
% le sabot.
position_sabot = 6;
% On va changer toutes les cartes qui n'ont pas la bonne hauteur, dans la
% limite de 3 cartes à échanger (ainsi on n'a pas le droit d'aller au-delà
% de la 8e carte du sabot).
for i = 1:5
    if main(i) ~= valeur_retenue && position_sabot <= 8
        main(i) = haut_sabot(position_sabot);
        position_sabot = position_sabot + 1;
    end
end
% On réutilise la sous-routine précédemment vue pour savoir combien de
% cartes de chaque hauteur a été tirée.
nombre = zeros (1, 13);
for i = 1:5
    nombre(main(i)) = nombre(main(i)) + 1;
end
% "score" est le nombre de fois que la hauteur la plus fréquente figure
% dans notre main. Noter que cette fois, c'est l'identité de cette hauteur
% qui ne nous intéresse pas.
score = max (nombre);
% On regarde si on a un brelan.
reponse = score >= 3;
return
end
