% La fonction Q1 évalue le volume de la boule par méthode de Monte-Carlo,
% en échantillonnant uniformément sur un pavé (en l'occurrence, le plus
% petit pavé contenant le support de la fonction indicatrice de la boule
% unité est [-1,+1]^6; c'est donc sur lui qu'on choisit d'échantillonner).
% On prend en argument le nombre "N" de simulations demandées.
function Q1 (N)
total = 0;
total_carres = 0;
% On va aussi chronométrer la fonction (en l'occurrence de la boucle
% principale), pour évaluer l'efficacité.
tic;
for i = 1:N
    % On simule un point x \in R^6 (représenté par un vecteur-ligne)
    % uniforme sur le pavé. Comme toutes les coordonnées se calculent de la
    % même manière, on utilise la syntaxe vectorielle de la fonction
    % "rand", qui permet d'obtenir un vecteur de lois Uniforme(0,1)
    % indépendantes.
    x = -1 + 2 * rand (1, 6);
    % On calcule l'indicatrice d'être la boule unité (en utilisant la
    % conversion implicite entre false/true et 0/1), à savoir le fait
    % d'avoir une norme inférieure à 1, ou encore une norme carrée
    % inférieure à 1. Notez qu'on peut calculer le carré de la norme
    % Euclidienne de x comme x multiplié par sa transposée.
    indicatrice = x * x' <= 1;
    total = total + indicatrice;
    % En fait ici total_carres est la même chose que total, mais il en
    % serait différemment pour une fonction à intégrer qui ne serait pas
    % une indicatrice.
    total_carres = total_carres + indicatrice * indicatrice;
end
% "t" est le temps de calcul (pour la boucle).
t = toc;
% Probabilité estimée qu'un point du pavé tombe dans la boule.
p = total / N;
% Écart-type (estimé) de l'estimateur p.
ect_p = sqrt (total_carres / N - p * p) / sqrt (N);
% Cependant, l'estimateur du volume de la boule, par rapport à p, est
% multiplié par le volume de la boule. Il faut donc adapter l'estimateur et
% son écart-type en conséquence. "vol_pave" désignera le volume (exact) du
% pavé.
vol_pave = 64;
% "vol_boule" est l'estimateur du volume de la boule.
vol_boule = vol_pave * p;
% "ect_volboule" est l'écart-type de vol_boule. Rappelons que, quand on
% multiplie une v.a. par un scalaire positif déterministe, son écart-type
% est multplié de la même façon.
ect_volboule = vol_pave * ect_p;
% On affiche l'estimateur et son intervalle de confiance. (La valeur qui
% majore en valeur absolue une loi normale standard à 90 % est 1.65).
fprintf ('J''estime le volume de la boule à %f ± %f ', ...
    vol_boule, 1.65 * ect_volboule);
fprintf ('(risque: 10 %%).\n')
% On affiche l'efficacité du calcul. Notez que celle-ci ne doit pas
% dépendre sensiblement du nombre de simulations effectuées.
fprintf ('L''efficacité du calcul ')
fprintf ('(ratio de sigma^-2 par le temps de calcul) est ici %f.\n', ...
    ect_volboule ^ -2 / t);
return
end
